﻿#region LICENSE
//
// Copyright (c) Microncode.com
// 
// This source is subject to the Microsoft Public License. 
// See https://opensource.org/licenses/ms-pl.html.
// All other rights reserved. 
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
//
#endregion

namespace ScreenRecorderLibSample
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Windows.Forms;
    using CSScreenRecorder;

    /// <summary>
    /// The main example of using the CSScreenRecorder library.
    /// </summary>

    public partial class frmMain : Form
    {
        //Create a new instance of the library
        ScreenRecorder ScreenRecorder;

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            //Add rotate style
            cboRotate.Items.Add("RotateNoneFlipNone");
            cboRotate.Items.Add("RotateNoneFlipXY");
            cboRotate.Items.Add("RotateNoneFlipY");
            
            //Set the rotate to none
            cboRotate.SelectedIndex = 0;

            // Set the spinning tool style
            foreach (var prop in typeof(Color).GetProperties())
            {
                if (Equals(prop.PropertyType.FullName, "System.Drawing.Color"))
                    this.cboSpinningToolColor.Items.Add(prop.Name);
            }

            cboSpinningToolStyle.DataSource = Enum.GetValues(typeof(System.Drawing.Drawing2D.DashStyle));

            cboSpinningToolColor.Text = "Red";
            cboSpinningToolStyle.Text = "Dot";

            using (ScreenRecorder = new CSScreenRecorder.ScreenRecorder("UserName", "RegKey"))
            {
                //Get the available codecs
                foreach (ScreenRecorder.Codecs Codec in ScreenRecorder.GetCodecs())
                {
                    Console.WriteLine(Codec.CodecName);
                    cboVideoCodecs.Items.Add(Codec.CodecName);
                }

                cboVideoCodecs.SelectedIndex = 1;

                #region LoadAudioProperties

                //Formats:
                cboAudioFormat.DataSource = ScreenRecorder.GetAudioFormats();
                cboAudioFormat.Text = "MP3";

                //Bitrate
                cboBitrate.DataSource = ScreenRecorder.GetAudioBitrates();
                cboBitrate.Text = "192";

                //Samplerate
                cboSamplerate.DataSource = ScreenRecorder.GetAudioSamplerates();
                cboSamplerate.Text = "44100";

                //Bit depth
                cboBits.DataSource = ScreenRecorder.GetAudioBits();
                cboBits.Text = "16";

                //Channels
                cboChannels.DataSource = ScreenRecorder.GetAudioChannels();
                cboChannels.Text = "2";

                //Recorder mode
                cboRecorderMode.DataSource = ScreenRecorder.GetAudioModes();
                cboRecorderMode.SelectedItem = ScreenRecorder.GetEnumValue(ScreenRecorder.AudioMode);

                #endregion

            }

            //Set the area to record to a full screen
            txtTop.Text = "0";
            txtLeft.Text = "0";

            //Full screen width and height
            txtWidth.Text = Screen.PrimaryScreen.Bounds.Width.ToString();
            txtHeight.Text = Screen.PrimaryScreen.Bounds.Height.ToString();

        }

        private void btnSpinningTool_Click(object sender, EventArgs e)
        {
            CSScreenRecorder.SpinningSelection SpinningSelection = new CSScreenRecorder.SpinningSelection();

            //In order to get your own UserName and RegKey please order a license.
            using (ScreenRecorder = new CSScreenRecorder.ScreenRecorder("UserName", "RegKey"))
            {
                //Set the spinning tool border color
                ScreenRecorder.SpinningToolColor = Color.FromName(cboSpinningToolColor.SelectedItem.ToString());

                //Set the spinning tool border size
                ScreenRecorder.SpinningToolBorderSize = Int32.Parse(txtSpinningToolBorder.Text);

                //Set the spinning tool border style
                ScreenRecorder.SpinningToolStyle = (System.Drawing.Drawing2D.DashStyle)cboSpinningToolStyle.SelectedIndex;

                //On Spinning Selection
                ScreenRecorder.OnSpinningSelection += (o, Cancel, Top, Left, Width, Height) =>
                {
                    //Spinning tool selection event:
                    if (Cancel)
                    {
                        txtLog.Text = txtLog.Text + DateTime.Now + ":Spinner tool canceled\r\n";
                    }
                    else
                    {
                        txtTop.Text = Top.ToString();
                        txtLeft.Text = Left.ToString();
                        txtWidth.Text = Width.ToString();
                        txtHeight.Text = Height.ToString();
                        txtLog.Text = txtLog.Text + DateTime.Now + ":Spinner tool selected - " + Top + " " + Left + " " + Width + " " + Height + "\r\n";
                    }

                };

                //Display the spinning tool
                SpinningSelection = ScreenRecorder.SpinningTool();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //Get active window
            CSScreenRecorder.ActiveWindow activewindow = new CSScreenRecorder.ActiveWindow();
            activewindow = ScreenRecorder.GetActiveWindow();

            //Set the active window coordinates
            txtTop.Text = activewindow.Top.ToString();
            txtLeft.Text = activewindow.Left.ToString();
            txtWidth.Text = activewindow.Width.ToString();
            txtHeight.Text = activewindow.Height.ToString();
        }

        private void btnActiveWindow_Click(object sender, EventArgs e)
        {
            if (!timer1.Enabled)
            {
                //Start get active window
                timer1.Enabled = true;
                btnActiveWindow.Text = "Stop";
            }
            else
            {
                //Stop get active window
                timer1.Enabled = false;
                btnActiveWindow.Text = "Get active window";
            }
        }

        private void cmdRecord_Click(object sender, EventArgs e)
        {
            using (ScreenRecorder = new CSScreenRecorder.ScreenRecorder("UserName", "RegKey"))
            {
                #region Area

                //Set the area to record
                ScreenRecorder.RecordScreenHeight = Int32.Parse(txtHeight.Text);
                ScreenRecorder.RecordScreenWidth = Int32.Parse(txtWidth.Text);
                ScreenRecorder.RecordScreenTop = Int32.Parse(txtTop.Text);
                ScreenRecorder.RecordScreenLeft = Int32.Parse(txtLeft.Text);

                #endregion

                #region Video

                //Set the frames rate
                ScreenRecorder.VideoFPS = 10;
                ScreenRecorder.VideoEncoderQuality = trcVideoEncoderQuality.Value;
                picPreview.SizeMode = PictureBoxSizeMode.StretchImage;

                //Set the video codec
                ScreenRecorder.VideoCodecIndex = cboVideoCodecs.SelectedIndex;

                #endregion Video

                #region Audio

                //Record audio?
                ScreenRecorder.AudioRecorder = chkRecordAudio.Checked;

                //Set the audio device index to record from:
                ScreenRecorder.AudioDeviceIndex = cboAudioSource.SelectedIndex;

                //[Optional: Default is WasapiLoopbackCapture]
                //The mode of the recording process. This can be WasapiLoopbackCapture(default), WasapiCapture or LineIn:
                ScreenRecorder.AudioMode = (CSScreenAudioRecorder.Mode)Enum.Parse(typeof(CSScreenAudioRecorder.Mode), cboRecorderMode.Text);

                //[Optional: Default is 44100]
                //Set the audio sample rate to record from the device and the destination file:
                ScreenRecorder.AudioSamplerate = (CSScreenAudioRecorder.Samplerate)Enum.Parse(typeof(CSScreenAudioRecorder.Samplerate), cboSamplerate.Text);
                //Or set by value
                //audioRecorder1.SamplerateVal = 48000;

                //[Optional: Default is 16]
                //Set the bit-depth to record from the device and the destination file:
                ScreenRecorder.AudioBits = (CSScreenAudioRecorder.Bits)Enum.Parse(typeof(CSScreenAudioRecorder.Bits), cboBits.Text);

                //[Optional: Default is 2]
                //Set the number of the channels to record from the device and the destination file:
                ScreenRecorder.AudioChannels = (CSScreenAudioRecorder.Channels)Enum.Parse(typeof(CSScreenAudioRecorder.Channels), cboChannels.Text);

                //[Optional: Default is MP3]
                //The audio format of the destination file, this can be MP3 or WAV(PCM).
                ScreenRecorder.AudioFormat = (CSScreenAudioRecorder.Format)Enum.Parse(typeof(CSScreenAudioRecorder.Format), cboAudioFormat.Text);

                //[Optional: Default is 192]
                //Set the bitrate of the MP3 encoder:
                ScreenRecorder.AudioBitrate = (CSScreenAudioRecorder.Bitrate)Enum.Parse(typeof(CSScreenAudioRecorder.Bitrate), cboBitrate.Text);
                //Or set by value
                //audioRecorder1.BitrateVal = 192;
                #endregion Audio

                #region Style

                //Track mouse move
                ScreenRecorder.TrackMouse = chkTrackMouse.Checked;

                //Set the rotate style
                ScreenRecorder.Rotate = CSScreenRecorder.RotateStyle.RotateNoneFlipNone;

                if (cboRotate.Text == "RotateNoneFlipXY")
                    ScreenRecorder.Rotate = CSScreenRecorder.RotateStyle.RotateNoneFlipXY;
                if (cboRotate.Text == "RotateNoneFlipY")
                    ScreenRecorder.Rotate = CSScreenRecorder.RotateStyle.RotateNoneFlipY;

                //Set the Watermark
                ScreenRecorder.WatermarkFont = new Font(new Font("Arial", 32), System.Drawing.FontStyle.Underline);
                ScreenRecorder.WatermarkText = txtWatermarkText.Text;

                #endregion Style

                #region Events

                //On start event
                ScreenRecorder.OnStartRecord += (o) =>
                {
                    lblPreview.Visible = false;
                    txtLog.Text = txtLog.Text + DateTime.Now + ":Start record\r\n";
                };

                //On pause record
                ScreenRecorder.OnPauseRecord += (o) =>
                {
                    txtLog.Text = txtLog.Text + DateTime.Now + ":Pause record\r\n";
                };

                //On unpause record
                ScreenRecorder.OnUnPauseRecord += (o) =>
                {
                    txtLog.Text = txtLog.Text + DateTime.Now + ":Unpause record\r\n";
                };

                //On progress (i = seconds)
                ScreenRecorder.OnProgress += (o, i) =>
                {
                    Console.WriteLine(i.ToString());
                };

                //On stop  record
                ScreenRecorder.OnStopRecord += (o) => { };

                //On done  record
                ScreenRecorder.OnDoneRecord += (o) =>
                {
                    lblPreview.Visible = true;
                    txtLog.Text = txtLog.Text + DateTime.Now + ":Done record\r\n";
                };

                //On preview (get the bmp image)
                ScreenRecorder.OnPreview += (o, bmp) =>
                {
                    //Preview
                    Bitmap pImage = new Bitmap(bmp);

                    picPreview.Image = pImage;
                };

                ScreenRecorder.OnError += (o, ErrorMessage, ErrorNumber) =>
                {
                    txtLog.Text = txtLog.Text + DateTime.Now + ":Error - " + ErrorMessage + " (" + ErrorNumber + ")\r\n";

                };

                #endregion

                //Set the file name
                txtFileName.Text = Application.StartupPath + "\\" + DateTime.Now.ToString("yyyy-MM-dd-HH-mm-ss") + ".avi";
                ScreenRecorder.FileName = txtFileName.Text;

                //Start to record
                ScreenRecorder.Record();
            }
        }

        private void cmdStopRec_Click(object sender, EventArgs e)
        {
            //Stop record
            ScreenRecorder.Stop();
        }

        private void cmdPauseIn_Click(object sender, EventArgs e)
        {
            //Pause record
            ScreenRecorder.Pause();
        }

        private void cmdUnPauseIn_Click(object sender, EventArgs e)
        {
            //UnPause record
            ScreenRecorder.UnPause();
        }

        /// <summary>
        /// Order a license.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdOrderLicense_Click(object sender, EventArgs e)
        {
            /// In order to get your own UserName and RegKey and to distribute this library
            /// with your own projects for commercial or any other purpose, please order a license at:
            /// https://www.microncode.com/developers/cs-screen-recorder/?cmd=order

            System.Diagnostics.Process.Start("https://www.microncode.com/developers/cs-screen-recorder/?cmd=order");
        }

        private void cboRecorderMode_SelectedIndexChanged(object sender, EventArgs e)
        {

            using (ScreenRecorder = new CSScreenRecorder.ScreenRecorder("UserName", "RegKey"))
            {
                //Get the selected capture mode:
                CSScreenAudioRecorder.Mode mode = (CSScreenAudioRecorder.Mode)Enum.Parse(typeof(CSScreenAudioRecorder.Mode), cboRecorderMode.Text);

                //Get the audio devices:
                cboAudioSource.DataSource = ScreenRecorder.GetDevices(mode);

                //Try to set the default device:
                int default_device_index = ScreenRecorder.GetDeviceDefaultIndex(mode);

                if (default_device_index != -1)
                    //Set the default device index:
                    cboAudioSource.SelectedIndex = default_device_index;
                //else
                    
                    //LineIn will always return -1:
                    //cboAudioSource.SelectedIndex = 0;

            }
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            #region SafeExit

            if(ScreenRecorder.RecordingState == ScreenRecorder.RecordingScreenState.Recording)

                ScreenRecorder.Stop();

            #endregion
        }
    }
}
